# Copyright (c) 2016 Boulder Real Time Technologies, Inc.
# All rights reserved 

# This software module is wholly owned by Boulder Real Time 
# Technologies, Inc. Any use of this software module without
# express written permission from Boulder Real Time Technologies,
# Inc. is prohibited.

import struct
import socket
import _stock

if __name__ == '__main__':
    import unittest
    import sys
    import os
    import warnings

    class Teststock(unittest.TestCase):

        def setUp(self):
            pass

        def tearDown(self):
            pass

        def test_pfget_string(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_string(pfptr, 'default_trace_schema')

            self.assertTrue(isinstance(val, str))

            _stock._pffree( pfptr )

        def test_pfget_int(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_int(pfptr, 'miniseed_record_size')

            self.assertTrue(isinstance(val, int))

            val = _stock._pfget_int(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_double(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_double(pfptr, 'samprate_tolerance')

            self.assertTrue(isinstance(val, float))

            val = _stock._pfget_double(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_size(self):
           
            (rc, pfptr) = _stock._pfread( 'orbserver' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_size(pfptr, 'ringsize')

            self.assertTrue(isinstance(val, float))

            val = _stock._pfget_size(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_time(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_time(pfptr, 'pf_revision_time')

            self.assertTrue(isinstance(val, float))

            val = _stock._pfget_time(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_boolean(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_boolean(pfptr, 'verbose_splicing')

            self.assertTrue(isinstance(val, bool))

            val = _stock._pfget_boolean(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_tbl(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_tbl(pfptr, 'waveform_types')

            self.assertTrue(isinstance(val, tuple))

            val = _stock._pfget_tbl(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfget_arr(self):
           
            (rc, pfptr) = _stock._pfread( 'rtexec' )

            self.assertEqual(rc, 0)

            val = _stock._pfget_arr(pfptr, 'Run')

            self.assertTrue(isinstance(val, dict))

            val = _stock._pfget_arr(pfptr, 'doesnotexist')

            self.assertEqual(val, None)

            _stock._pffree( pfptr )

        def test_pfread(self):
           
            (rc, pfptr) = _stock._pfread('rtexec')

            self.assertTrue(rc == 0)

            _stock._pffree( pfptr )

        def test_pfin(self):
           
            (rc, pfptr) = _stock._pfin( os.environ['ANTELOPE'] + '/data/pf/rtexec.pf' )

            self.assertTrue(rc == 0)

            _stock._pffree( pfptr )

        def test_pfupdate(self):
           
            (rc, pfptr) = _stock._pfread('rtexec')

            (rc, pfptr) = _stock._pfupdate('rtexec', pfptr)

            self.assertTrue(rc == 0 or rc == 1)

            _stock._pffree( pfptr )

        def test_pfget(self):

            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)
           
            (rc, pfptr2) = _stock._pfread( 'orbserver' )

            self.assertEqual(rc, 0)
           
            (rc, pfptr3) = _stock._pfread( 'rtexec' )

            self.assertEqual(rc, 0)
           
            (rc, pfptr4) = _stock._pfread( 'orbserver_names' )

            self.assertEqual(rc, 0)
           
            val = _stock._pfget(pfptr, 'default_trace_schema')
            self.assertTrue(isinstance(val, str))

            val = _stock._pfget(pfptr, 'miniseed_record_size')
            self.assertTrue(isinstance(val, str))

            val = _stock._pfget(pfptr, 'samprate_tolerance')
            self.assertTrue(isinstance(val, str))

            val = _stock._pfget(pfptr, 'waveform_types')
            self.assertTrue(isinstance(val, tuple))

            val = _stock._pfget(pfptr2, 'ringsize')
            self.assertTrue(isinstance(val, str))

            val = _stock._pfget(pfptr3, 'Run')
            self.assertTrue(isinstance(val, dict))

            val = _stock._pfget(pfptr4, None)
            self.assertTrue(isinstance(val, dict))

            _stock._pffree( pfptr )
            _stock._pffree( pfptr2 )
            _stock._pffree( pfptr3 )
            _stock._pffree( pfptr4 )

        def test_pffiles(self):

            files = _stock._pffiles('rtexec', False)

            self.assertTrue(isinstance(files,tuple))

            warnings.filterwarnings('ignore', 'Attempt to coerce non-Boolean.*', RuntimeWarning)

            warnings.resetwarnings()

        @unittest.skip("TODO: Find a way of passing python file handles to C extensions in py3")
        def test_pfout(self):

            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)

            fp = open('/tmp/trdefaults_test.pf', 'w')

            rc = _stock._pfout(pfptr, fp)

            fp.close()

            self.assertEqual(rc, 0)

            _stock._pffree( pfptr )

        def test_pfwrite(self):

            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)
           
            rc = _stock._pfwrite(pfptr, '/tmp/trdefaults_test.pf')

            self.assertEqual(rc, 0)

            _stock._pffree( pfptr )

        def test_pf2string(self):

            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)
           
            val = _stock._pf2string(pfptr)

            self.assertTrue(isinstance(val, str))

            _stock._pffree( pfptr )

        def test_pfnew(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            _stock._pffree( pfptr )

        def test_pfdup(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            _stock._pfput( 'myint', 3, pfptr )

            pfptr2 = _stock._pfdup( pfptr )

            _stock._pffree( pfptr )
            _stock._pffree( pfptr2 )

        def test_pfcompile(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            rc = _stock._pfcompile('myint 3', pfptr)

            self.assertEqual(rc, 0)

        def test_pfrequire(self):

            rc = _stock._pfrequire('rtexec', '11/15/1996')

            self.assertEqual(rc, 0)

        def test_pfdel(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            _stock._pfcompile('myint 3', pfptr)
            _stock._pfcompile('myreal 3.4', pfptr)
            _stock._pfcompile('mystr pi', pfptr)

            pfptrdeleted = _stock._pfdel(pfptr, 'mystr')

            _stock._pffree( pfptrdeleted )

        def test_pfkeys(self):

            (rc, pfptr ) = _stock._pfread( 'rtexec' )

            keys = _stock._pfkeys( pfptr )

            self.assertTrue(isinstance(keys,tuple))

            try:

                val = keys.index('Processes')

            except ValueError:

                self.fail( "No Processes table found in rtexec.pf by pfkeys()" )
              
        def test_pfput(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            _stock._pfput( 'myint', 3, pfptr )
            _stock._pfput( 'myreal', 3.4, pfptr )
            _stock._pfput( 'mystr', 'pi', pfptr )
            _stock._pfput( 'mytbl', ( 'a', 'b', 'c' ), pfptr )
            _stock._pfput( 'myarr', { 'ai' : 13, 'bi' : 25 }, pfptr )

            val = _stock._pf2string( pfptr )

            self.assertEqual( val, 'myarr\t&Arr{\nai\t13\nbi\t25\n}\nmyint\t3\nmyreal\t3.400000\nmystr\tpi\nmytbl\t&Tbl{\na\nb\nc\n}\n' )

        def test_pffree(self):

            pfptr = _stock._pfnew( _stock.PFARR )

            _stock._pffree(pfptr)

        def test_pf2xml(self):
           
            (rc, pfptr) = _stock._pfread( 'trdefaults' )

            self.assertEqual(rc, 0)
           
            val = _stock._pf2xml(pfptr, _stock.PFXML_STRONG, None, None)

            self.assertTrue(isinstance(val, str))

            _stock._pffree(pfptr)

        def test_is_epoch_string(self):

            val = _stock._is_epoch_string('6/6/2011 15:31 PM')

            self.assertTrue(val)

            ( val, e ) = _stock._is_epoch_string('Not a time string')

            self.assertTrue(not val)
            self.assertTrue(isinstance(e,float))

        def test_strtime(self):

            e = 0
            t = _stock._strtime(e)

            self.assertEqual(t,' 1/01/1970   0:00:00.000')

        def test_strydtime(self):

            e = 0
            t = _stock._strydtime(e)

            self.assertEqual(t,' 1/01/1970 (001)  0:00:00.000')

        def test_strdate(self):

            e = 0
            t = _stock._strdate(e)

            self.assertEqual(t,' 1/01/1970')

        def test_strlocaldate(self):

            e = 0
            t = _stock._strlocaldate(e)

            self.assertTrue(isinstance(t, str))

        def test_strlocaltime(self):

            e = 0
            t = _stock._strlocaltime(e)

            self.assertTrue(isinstance(t, str))

        def test_strlocalydtime(self):

            e = 0
            t = _stock._strlocalydtime(e)

            self.assertTrue(isinstance(t, str))

        def test_strtdelta(self):

            e = 0
            t = _stock._strtdelta(e)

            self.assertTrue(t == ' 0 microseconds      ' or t == '  0 seconds          ')

        def test_str2epoch(self):

            s = '1/1/2000 14:00'

            e = _stock._str2epoch(s)

            self.assertEqual(e, 946735200)

        def test_epoch2str(self):

            e = 946735200

            s = _stock._epoch2str(e, "%m/%d/%Y %H:%M", None)
            self.assertEqual(s, '01/01/2000 14:00')

            z = _stock._epoch2str(e, "%m/%d/%Y %H:%M %Z", "US/Alaska")
            self.assertEqual(z, '01/01/2000 05:00 AKST')

        def test_epoch(self):

            yd = 2000001
            e = _stock._epoch(yd)

            self.assertEqual(e, 946684800)

        def test_yearday(self):

            e = 946684800
            yd = _stock._yearday(e)

            self.assertEqual(yd, 2000001)

        def test_now(self):

            e = _stock._now()

            self.assertTrue(isinstance(e,float))

        def test_grname(self):

            name = _stock._grname(65, -150)

            self.assertEqual(name, "NORTHERN ALASKA" )

        def test_srname(self):

            name = _stock._srname(65, -150)

            self.assertEqual(name, "NORTHEASTERN ASIA, NORTHERN ALASKA TO GREENLAND" )

        def test_grn(self):

            num = _stock._grn(65, -150)

            self.assertEqual(num, 676)

        def test_srn(self):

            num = _stock._srn(65, -150)

            self.assertEqual(num, 42)

        def test_datafile(self):

            apath = _stock._datafile( 'PATH', 'ls' )

            self.assertEqual( apath, '/bin/ls' )

        def test_datapath(self):

            apath = _stock._datapath( 'PATH', None, 'ls', None )

            self.assertEqual( apath, '/bin/ls' )

        def test_legit_ip(self):

            addr = socket.inet_aton('127.0.0.1')
            ip = struct.unpack('=L', addr)[0]
            (rc, rstrict) = _stock._legit_ip( 'orbserver', ip )

            self.assertEqual( rc, True )
            self.assertEqual( rstrict, None )

            addr = socket.inet_aton('23.31.71.129')
            ip = struct.unpack('=L', addr)[0]
            (rc, rstrict) = _stock._legit_ip( 'orbserver', ip )

            self.assertEqual( rc, True )
            self.assertEqual( rstrict, 'readonly' )

            addr = socket.inet_aton('4.2.2.4')
            ip = struct.unpack('=L', addr)[0]
            (rc, rstrict) = _stock._legit_ip( 'orbserver', ip )

            self.assertEqual( rc, False )
            self.assertEqual( rstrict, None )
        
        def test_yesno(self):

            self.assertEqual( _stock._yesno( "yes" ), -1 )
            self.assertEqual( _stock._yesno( "Yes" ), -1 )
            self.assertEqual( _stock._yesno( "YES" ), -1 )
            self.assertEqual( _stock._yesno( "ok" ), -1 )
            self.assertEqual( _stock._yesno( "y" ), -1 )
            self.assertEqual( _stock._yesno( "1" ), -1 )
            self.assertEqual( _stock._yesno( "t" ), -1 )
            self.assertEqual( _stock._yesno( "on" ), -1 )

            self.assertEqual( _stock._yesno( "no" ), 0 )
            self.assertEqual( _stock._yesno( "No" ), 0 )
            self.assertEqual( _stock._yesno( "NO" ), 0 )
            self.assertEqual( _stock._yesno( "n" ), 0 )
            self.assertEqual( _stock._yesno( "false" ), 0 )
            self.assertEqual( _stock._yesno( "0" ), 0 )
            self.assertEqual( _stock._yesno( "off" ), 0 )
            self.assertEqual( _stock._yesno( "f" ), 0 )

            self.assertEqual( _stock._yesno( "isn't a valid yesno string" ), 1 )

    unittest.main()
